/*
 * vanadiel-clock.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Contains methods for obtaining the current Vana'diel based on the
 * current system time.
 */

// Name of Vana'diel days - note that these are simple text phrases and not English.
// This is so that you can later localize them.

var vanaDays = [ "fire", "earth", "water", "wind", "ice", "thunder", "light", "dark" ];

// Name of moon phases, same note as the days
var moonPhases = [ "new", "waxingcrescent", "firstquarter", "waxinggibbous", "full", "waninggibbous", "lastquarter", "waningcrescent" ];

// Offset in milliseconds between the starting Vana'diel epoch and the UNIX epoch
var vanadielOffset = 0;

{
    // Calculate the offset to convert to Vana'diel time for the current time zone

    // I, unfortunately, have no idea who originally came up with the following.
    // Note that it has since been converted to GMT.
    
    // Earth time:       2002/06/23 15:00:00 GMT
    // Vana'diel time:   0898/02/01 00:00:00 (VST?)

    var vanaTime  = (898 * 360 + 30) * 24 * 60 * 60 * 1000 / 25;
    var baseDay = new Date();
    baseDay.setUTCFullYear(2002, 5, 23); // Set date to 2002-06-23
    baseDay.setUTCHours(15, 0, 0, 0);    // Set time to 15:00:00.0000

    vanadielOffset = baseDay.getTime() - vanaTime;
}

/*
 * Creates a new Vana'diel Date.
 * If time is a non-null integer, than it will be used as the current Earth date
 * to calculate Vana'diel time for.
 *
 * Once you've created the object, you can access the following properties on it
 * to get the current Vana'diel time:
 *
 *   day         - current number of days since the Vana'diel epoch - you
 *                 probably want dayMonth
 *   year        - current Vana'diel Year
 *   month       - current Vana'diel Month
 *   dayMonth    - current day of the month
 *   dayWeek     - day of the week - convert to an elemental string with the
 *                 vanaDays array or create your own
 *   hour        - current Vana'diel hour
 *   minute      - current Vana'diel minutes
 *   second      - current Vana'diel seconds
 *   moonPhase   - moon phase "counter", more on this later
 *   moonPercent - moon phase as a percentage (always positive)
 *   waxing      - true if the moon is currently waxing
 *   basicMoonPhase - a number 0 through 7 marking the simple moon phase
 *                    positions - use the moonPhases array to translate this
 *                    to a specific phase
 *
 * The "moonPhase" variable is a number from 0 to 83 that represents which day
 * in the moon phase we are currently in.
 *
 * To see how the "phase counter" stacks up to a percent, run the following
 * Perl 1-liner:
 *
 * perl -e 'for($p=0;$p<84;$p++){print "$p = ".int((abs((42-$p)*100/42)+0.5))."\% (wa".($p<42?"n":"x")."ing)\n";}'
 *
 * Simple, huh?
 *
 * === MOON PHASES ===
 * 80- 2 == 90%(Wax)  - 95%(Wane) - Full
 *  3-16 == 93%(Wane) - 62%(Wane) - Waning Gibbous
 * 17-23 == 60%(Wane) - 45%(Wane) - Last Quarter
 * 24-37 == 43%(Wane) - 12%(Wane) - Waning Crescent
 * 38-44 == 10%(Wane) -  5%(Wax)  - New Moon
 * 45-58 ==  7%(Wax)  - 38%(Wax)  - Waxing Crescent
 * 59-65 == 40%(Wax)  - 55%(Wax)  - First Quarter
 * 66-79 == 57%(Wax)  - 88%(Wax)  - Waxing Gibbous
 */
function VanadielDate(time) {
    // Vana'diel time runs 25 times faster than our time - this is the time
    // in "Vana'diel milliseconds" actually.
    if (time == null) {
	time = new Date().getTime();
    } else if (time instanceof Date) {
	time = time.getTime();
    }
    this.time = (time - vanadielOffset) * 25;
    // We only display the day, hour, and minute in the little clock
    this.day = Math.floor(this.time / 86400000);
    this.year = Math.floor(this.day / 360);
    this.month = Math.floor((this.day % 360) / 30) + 1;
    this.dayMonth = (this.day % 30) + 1;
    this.dayWeek = this.day % 8;
    this.hour = Math.floor(this.time / 3600000) % 24;
    this.minute = Math.floor(this.time / 60000) % 60;
    this.second = Math.floor(this.time / 1000) % 60;
    // calculate the current moon phase - unlike the current
    // day of the week, this isn't nice and happily rooted at 0
    // moonphase = 0 - full moon
    // moonphase = 42 - new moon
    this.moonPhase = (this.day + 26) % 84;
    // translate that value into a percentage value
    this.moonPercent = ((42 - this.moonPhase) * 100) / 42;
    // negative values are "waxing" and backwards
    this.waxing = false;
    if (this.moonPercent < 0) {
	this.waxing = true;
	this.moonPercent = Math.abs(this.moonPercent);
    }
    // Now make it an interger
    this.moonPercent = Math.floor(this.moonPercent + 0.5);
    // Translate the phase into one of the eight phases.
    this.basicMoonPhase = 4; // full
    if (this.moonPhase < 3) {
	this.basicMoonPhase = 4; // full
    } else if (this.moonPhase < 17) {
	this.basicMoonPhase = 5; // waning gibbous
    } else if (this.moonPhase < 24) {
	this.basicMoonPhase = 6; // last quarter
    } else if (this.moonPhase < 38) {
	this.basicMoonPhase = 7; // waning crescent
    } else if (this.moonPhase < 45) {
	this.basicMoonPhase = 0; // new
    } else if (this.moonPhase < 59) {
	this.basicMoonPhase = 1; // waxing crescent
    } else if (this.moonPhase < 66) {
	this.basicMoonPhase = 2; // first quarter
    } else if (this.moonPhase < 80) {
	this.basicMoonPhase = 3; // waxing gibbous
    } else {
	this.basicMoonPhase = 4; // full
    }
}

